@@ -20,11 +20,9 @@
 #ifndef AUDACIOUS_I18N_H
 #define AUDACIOUS_I18N_H
 
-#include <glib/gstrfuncs.h>
 #include <libintl.h>
 
 #define _(String) dgettext (PACKAGE, String)
-#define Q_(String) g_strip_context ((String), dgettext (PACKAGE, String))
 #ifdef gettext_noop
 #define N_(String) gettext_noop (String)
 #else
