@@ -19,15 +19,21 @@
  * using our public API to be a derived work.
  */
 
+/* Header for all those files that have just one or two public identifiers. */
+
 #ifndef _AUDACIOUS_MAIN_H
 #define _AUDACIOUS_MAIN_H
 
-#include <glib.h>
+#include <libaudcore/core.h>
 
 /* adder.c */
 void adder_init (void);
 void adder_cleanup (void);
 
+/* art.c */
+void art_init (void);
+void art_cleanup (void);
+
 /* chardet.c */
 void chardet_init (void);
 
@@ -36,9 +42,12 @@ void config_load (void);
 void config_save (void);
 void config_cleanup (void);
 
+/* history.c */
+void history_cleanup (void);
+
 /* main.c */
-extern gboolean headless;
-gboolean do_autosave (void);
+extern bool_t headless;
+bool_t do_autosave (void);
 
 /* mpris-signals.c */
 void mpris_signals_init (void);
@@ -50,4 +59,7 @@ void signals_init (void);
 /* smclient.c */
 void smclient_init (void);
 
+/* ui_albumart.c */
+char * get_associated_image_file (const char * filename);
+
 #endif
