@@ -2,21 +2,19 @@
  * queue-manager.c
  * Copyright 2011 John Lindgren
  *
- * This file is part of Audacious.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
  *
- * Audacious is free software: you can redistribute it and/or modify it under
- * the terms of the GNU General Public License as published by the Free Software
- * Foundation, version 2 or version 3 of the License.
+ * 1. Redistributions of source code must retain the above copyright notice,
+ *    this list of conditions, and the following disclaimer.
  *
- * Audacious is distributed in the hope that it will be useful, but WITHOUT ANY
- * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
- * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+ * 2. Redistributions in binary form must reproduce the above copyright notice,
+ *    this list of conditions, and the following disclaimer in the documentation
+ *    provided with the distribution.
  *
- * You should have received a copy of the GNU General Public License along with
- * Audacious. If not, see <http://www.gnu.org/licenses/>.
- *
- * The Audacious team does not consider modular code linking to Audacious or
- * using our public API to be a derived work.
+ * This software is provided "as is" and without any warranty, express or
+ * implied. In no event shall the authors be liable for any damages arising from
+ * the use of this software.
  */
 
 #include <gdk/gdkkeysyms.h>
@@ -28,6 +26,7 @@
 #include <libaudcore/hook.h>
 
 #include "config.h"
+#include "init.h"
 #include "libaudgui.h"
 #include "list.h"
 
@@ -40,10 +39,10 @@ enum {
 static GtkWidget * qm_win;
 static GtkWidget * qm_list;
 
-static void get_value (void * user, gint row, gint column, GValue * value)
+static void get_value (void * user, int row, int column, GValue * value)
 {
-    gint list = aud_playlist_get_active ();
-    gint entry = aud_playlist_queue_get_entry (list, row);
+    int list = aud_playlist_get_active ();
+    int entry = aud_playlist_queue_get_entry (list, row);
 
     switch (column)
     {
@@ -51,42 +50,43 @@ static void get_value (void * user, gint row, gint column, GValue * value)
         g_value_set_int (value, 1 + entry);
         break;
     case COLUMN_TITLE:;
-        gchar * title = aud_playlist_entry_get_title (list, entry, TRUE);
-        g_value_take_string (value, title);
+        char * title = aud_playlist_entry_get_title (list, entry, TRUE);
+        g_value_set_string (value, title);
+        str_unref (title);
         break;
     }
 }
 
-static gboolean get_selected (void * user, gint row)
+static bool_t get_selected (void * user, int row)
 {
-    gint list = aud_playlist_get_active ();
+    int list = aud_playlist_get_active ();
     return aud_playlist_entry_get_selected (list, aud_playlist_queue_get_entry (list, row));
 }
 
-static void set_selected (void * user, gint row, gboolean selected)
+static void set_selected (void * user, int row, bool_t selected)
 {
-    gint list = aud_playlist_get_active ();
+    int list = aud_playlist_get_active ();
     aud_playlist_entry_set_selected (list, aud_playlist_queue_get_entry (list, row), selected);
 }
 
-static void select_all (void * user, gboolean selected)
+static void select_all (void * user, bool_t selected)
 {
-    gint list = aud_playlist_get_active ();
-    gint count = aud_playlist_queue_count (list);
+    int list = aud_playlist_get_active ();
+    int count = aud_playlist_queue_count (list);
 
-    for (gint i = 0; i < count; i ++)
+    for (int i = 0; i < count; i ++)
         aud_playlist_entry_set_selected (list, aud_playlist_queue_get_entry (list, i), selected);
 }
 
-static void shift_rows (void * user, gint row, gint before)
+static void shift_rows (void * user, int row, int before)
 {
-    GArray * shift = g_array_new (FALSE, FALSE, sizeof (gint));
-    gint list = aud_playlist_get_active ();
-    gint count = aud_playlist_queue_count (list);
+    GArray * shift = g_array_new (FALSE, FALSE, sizeof (int));
+    int list = aud_playlist_get_active ();
+    int count = aud_playlist_queue_count (list);
 
-    for (gint i = 0; i < count; i ++)
+    for (int i = 0; i < count; i ++)
     {
-        gint entry = aud_playlist_queue_get_entry (list, i);
+        int entry = aud_playlist_queue_get_entry (list, i);
 
         if (aud_playlist_entry_get_selected (list, entry))
         {
@@ -99,8 +99,8 @@ static void shift_rows (void * user, gint row, gint before)
 
     aud_playlist_queue_delete_selected (list);
 
-    for (gint i = 0; i < shift->len; i ++)
-        aud_playlist_queue_insert (list, before + i, g_array_index (shift, gint, i));
+    for (int i = 0; i < shift->len; i ++)
+        aud_playlist_queue_insert (list, before + i, g_array_index (shift, int, i));
 
     g_array_free (shift, TRUE);
 }
@@ -114,12 +114,12 @@ static const AudguiListCallbacks callbacks = {
 
 static void remove_selected (void)
 {
-    gint list = aud_playlist_get_active ();
-    gint count = aud_playlist_queue_count (list);
+    int list = aud_playlist_get_active ();
+    int count = aud_playlist_queue_count (list);
 
-    for (gint i = 0; i < count; )
+    for (int i = 0; i < count; )
     {
-        gint entry = aud_playlist_queue_get_entry (list, i);
+        int entry = aud_playlist_queue_get_entry (list, i);
 
         if (aud_playlist_entry_get_selected (list, entry))
         {
@@ -159,9 +159,9 @@ static void destroy_cb (void)
     qm_list = NULL;
 }
 
-static gboolean keypress_cb (GtkWidget * widget, GdkEventKey * event)
+static bool_t keypress_cb (GtkWidget * widget, GdkEventKey * event)
 {
-    if (event->keyval == GDK_A && (event->state && GDK_CONTROL_MASK))
+    if (event->keyval == GDK_A && (event->state & GDK_CONTROL_MASK))
         select_all (NULL, TRUE);
     else if (event->keyval == GDK_Delete)
         remove_selected ();
@@ -173,7 +173,7 @@ static gboolean keypress_cb (GtkWidget * widget, GdkEventKey * event)
     return TRUE;
 }
 
-static void response_cb (GtkDialog * dialog, gint response)
+static void response_cb (GtkDialog * dialog, int response)
 {
     switch (response)
     {
@@ -185,7 +185,6 @@ static void response_cb (GtkDialog * dialog, gint response)
         break;
     }
 }
-
 void audgui_queue_manager_show (void)
 {
     if (qm_win)
@@ -197,18 +196,19 @@ void audgui_queue_manager_show (void)
     qm_win = gtk_dialog_new_with_buttons (_("Queue Manager"), NULL, 0,
      GTK_STOCK_REMOVE, RESPONSE_REMOVE, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
      NULL);
-    gtk_window_set_default_size ((GtkWindow *) qm_win, 250, 250);
+    gtk_window_set_default_size ((GtkWindow *) qm_win, 400, 250);
 
     GtkWidget * vbox = gtk_dialog_get_content_area ((GtkDialog *) qm_win);
 
     GtkWidget * scrolled = gtk_scrolled_window_new (NULL, NULL);
+    gtk_scrolled_window_set_shadow_type ((GtkScrolledWindow *) scrolled, GTK_SHADOW_IN);
     gtk_box_pack_start ((GtkBox *) vbox, scrolled, TRUE, TRUE, 0);
 
-    gint count = aud_playlist_queue_count (aud_playlist_get_active ());
+    int count = aud_playlist_queue_count (aud_playlist_get_active ());
     qm_list = audgui_list_new (& callbacks, NULL, count);
     gtk_tree_view_set_headers_visible ((GtkTreeView *) qm_list, FALSE);
-    audgui_list_add_column (qm_list, NULL, 0, G_TYPE_INT, FALSE);
-    audgui_list_add_column (qm_list, NULL, 1, G_TYPE_STRING, TRUE);
+    audgui_list_add_column (qm_list, NULL, 0, G_TYPE_INT, 7);
+    audgui_list_add_column (qm_list, NULL, 1, G_TYPE_STRING, -1);
     gtk_container_add ((GtkContainer *) scrolled, qm_list);
 
     hook_associate ("playlist activate", update_hook, NULL);
@@ -220,3 +220,9 @@ void audgui_queue_manager_show (void)
 
     gtk_widget_show_all (qm_win);
 }
+
+void audgui_queue_manager_cleanup (void)
+{
+    if (qm_win)
+        gtk_widget_destroy (qm_win);
+}
