@@ -1,26 +1,23 @@
-/*  Audacious - Cross-platform multimedia player
- *  Copyright (C) 2005-2011  Audacious development team.
+/*
+ * ui_jumptotrack.c
+ * Copyright 1998-2003 XMMS development team
+ * Copyright 2003-2004 BMP development team
+ * Copyright 2007-2011 Yoshiki Yazawa and John Lindgren
  *
- *  BMP - Cross-platform multimedia player
- *  Copyright (C) 2003-2004  BMP development team.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; under version 3 of the License.
  *
- *  Based on XMMS:
- *  Copyright (C) 1998-2003  XMMS development team.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; under version 3 of the License.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses>.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses>.
- *
- *  The Audacious team does not consider modular code linking to
- *  Audacious or using our public API to be a derived work.
+ * The Audacious team does not consider modular code linking to
+ * Audacious or using our public API to be a derived work.
  */
 
 #include <gdk/gdkkeysyms.h>
@@ -34,7 +31,8 @@
 #include <libaudcore/hook.h>
 
 #include "config.h"
-#include "icons-stock.h"
+#include "libaudgui.h"
+#include "list.h"
 #include "ui_jumptotrack_cache.h"
 
 static void update_cb (void * data, void * user);
@@ -42,8 +40,9 @@ static void activate_cb (void * data, void * user);
 
 static GtkWidget *jump_to_track_win = NULL;
 static JumpToTrackCache* cache = NULL;
+static const GArray * search_matches;
 static GtkWidget * treeview, * filter_entry, * queue_button;
-static gboolean watching = FALSE;
+static bool_t watching = FALSE;
 
 void
 audgui_jump_to_track_hide(void)
@@ -63,31 +62,36 @@ audgui_jump_to_track_hide(void)
         ui_jump_to_track_cache_free (cache);
         cache = NULL;
     }
+
+    search_matches = NULL;
 }
 
-static gint get_selected_entry (void)
+static int get_selected_entry (void)
 {
-    g_return_val_if_fail (treeview, -1);
+    g_return_val_if_fail (treeview && search_matches, -1);
 
     GtkTreeModel * model = gtk_tree_view_get_model ((GtkTreeView *) treeview);
     GtkTreeSelection * selection = gtk_tree_view_get_selection ((GtkTreeView *) treeview);
     GtkTreeIter iter;
-    gint entry;
 
     if (! gtk_tree_selection_get_selected (selection, NULL, & iter))
         return -1;
 
-    gtk_tree_model_get (model, & iter, 0, & entry, -1);
-    return entry - 1;
+    GtkTreePath * path = gtk_tree_model_get_path (model, & iter);
+    int row = gtk_tree_path_get_indices (path)[0];
+    gtk_tree_path_free (path);
+
+    g_return_val_if_fail (row >= 0 && row < search_matches->len, -1);
+    return g_array_index (search_matches, int, row);
 }
 
 static void do_jump (void)
 {
-    gint entry = get_selected_entry ();
+    int entry = get_selected_entry ();
     if (entry < 0)
         return;
 
-    gint playlist = aud_playlist_get_active ();
+    int playlist = aud_playlist_get_active ();
     aud_playlist_set_playing (playlist);
     aud_playlist_set_position (playlist, entry);
     aud_drct_play ();
@@ -96,7 +100,7 @@ static void do_jump (void)
         audgui_jump_to_track_hide();
 }
 
-static void update_queue_button (gint entry)
+static void update_queue_button (int entry)
 {
     g_return_if_fail (queue_button);
 
@@ -118,14 +122,16 @@ static void update_queue_button (gint entry)
 
 static void do_queue (void)
 {
-    gint entry = get_selected_entry ();
+    int playlist = aud_playlist_get_active ();
+    int entry = get_selected_entry ();
     if (entry < 0)
         return;
 
-    if (aud_drct_pq_is_queued (entry))
-        aud_drct_pq_remove (entry);
+    int queued = aud_playlist_queue_find_entry (playlist, entry);
+    if (queued >= 0)
+        aud_playlist_queue_delete (playlist, queued, 1);
     else
-        aud_drct_pq_add (entry);
+        aud_playlist_queue_insert (playlist, -1, entry);
 
     update_queue_button (entry);
 }
@@ -135,7 +141,7 @@ static void selection_changed (void)
     update_queue_button (get_selected_entry ());
 }
 
-static gboolean keypress_cb (GtkWidget * widget, GdkEventKey * event)
+static bool_t keypress_cb (GtkWidget * widget, GdkEventKey * event)
 {
     if (event->keyval == GDK_Escape)
     {
@@ -153,31 +159,11 @@ static void fill_list (void)
     if (! cache)
         cache = ui_jump_to_track_cache_new();
 
-    GtkListStore * store = (GtkListStore *) gtk_tree_view_get_model ((GtkTreeView *) treeview);
-    GtkTreeIter iter;
+    search_matches = ui_jump_to_track_cache_search (cache, gtk_entry_get_text
+     ((GtkEntry *) filter_entry));
 
-    gtk_list_store_clear(store);
-
-    const GArray * search_matches = ui_jump_to_track_cache_search (cache,
-     gtk_entry_get_text ((GtkEntry *) filter_entry));
-
-    gint playlist = aud_playlist_get_active ();
-
-    for (gint i = 0; i < search_matches->len; i ++)
-    {
-        gint entry = g_array_index (search_matches, gint, i);
-        gchar * title = aud_playlist_entry_get_title (playlist, entry, TRUE);
-        if (! title)
-            continue;
-
-        gtk_list_store_append(store, &iter);
-        gtk_list_store_set (store, & iter, 0, 1 + entry, 1, title, -1);
-        g_free (title);
-    }
-
-    if (gtk_tree_model_get_iter_first ((GtkTreeModel *) store, & iter))
-        gtk_tree_selection_select_iter (gtk_tree_view_get_selection
-         ((GtkTreeView *) treeview), & iter);
+    audgui_list_delete_rows (treeview, 0, audgui_list_row_count (treeview));
+    audgui_list_insert_rows (treeview, 0, search_matches->len);
 }
 
 static void clear_cb (GtkWidget * widget)
@@ -226,24 +212,50 @@ static void activate_cb (void * data, void * user)
     update_cb (GINT_TO_POINTER (PLAYLIST_UPDATE_STRUCTURE), NULL);
 }
 
-static void toggle_button_cb (GtkToggleButton * toggle, const gchar * setting)
+static void toggle_button_cb (GtkToggleButton * toggle, const char * setting)
 {
     aud_set_bool ("audgui", setting, gtk_toggle_button_get_active (toggle));
 }
 
-static gboolean delete_cb (void)
+static bool_t delete_cb (void)
 {
     audgui_jump_to_track_hide ();
     return TRUE;
 }
 
+static void list_get_value (void * user, int row, int column, GValue * value)
+{
+    g_return_if_fail (search_matches);
+    g_return_if_fail (column >= 0 && column < 2);
+    g_return_if_fail (row >= 0 && row < search_matches->len);
+
+    int playlist = aud_playlist_get_active ();
+    int entry = g_array_index (search_matches, int, row);
+
+    switch (column)
+    {
+    case 0:
+        g_value_set_int (value, 1 + entry);
+        break;
+    case 1:;
+        char * title = aud_playlist_entry_get_title (playlist, entry, TRUE);
+        g_return_if_fail (title);
+        g_value_set_string (value, title);
+        str_unref (title);
+        break;
+    }
+}
+
+static const AudguiListCallbacks callbacks = {
+ .get_value = list_get_value};
+
 static void create_window (void)
 {
     jump_to_track_win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     gtk_window_set_type_hint(GTK_WINDOW(jump_to_track_win),
                              GDK_WINDOW_TYPE_HINT_DIALOG);
 
-    gtk_window_set_title(GTK_WINDOW(jump_to_track_win), _("Jump to Track"));
+    gtk_window_set_title(GTK_WINDOW(jump_to_track_win), _("Jump to Song"));
 
     g_signal_connect (jump_to_track_win, "key_press_event", (GCallback) keypress_cb, NULL);
     g_signal_connect (jump_to_track_win, "delete-event", (GCallback) delete_cb, NULL);
@@ -254,24 +266,9 @@ static void create_window (void)
     GtkWidget * vbox = gtk_vbox_new (FALSE, 5);
     gtk_container_add(GTK_CONTAINER(jump_to_track_win), vbox);
 
-    GtkListStore * jtf_store = gtk_list_store_new (2, G_TYPE_INT, G_TYPE_STRING);
-    treeview = gtk_tree_view_new_with_model(GTK_TREE_MODEL(jtf_store));
-    g_object_unref(jtf_store);
-
-    GtkTreeViewColumn * column = gtk_tree_view_column_new ();
-    gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(treeview), FALSE);
-    gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
-
-    GtkCellRenderer * renderer = gtk_cell_renderer_text_new ();
-    gtk_tree_view_column_pack_start(column, renderer, FALSE);
-    gtk_tree_view_column_set_attributes(column, renderer, "text", 0, NULL);
-
-    renderer = gtk_cell_renderer_text_new();
-    gtk_tree_view_column_pack_start(column, renderer, FALSE);
-    gtk_tree_view_column_set_attributes(column, renderer, "text", 1, NULL);
-    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
-
-    gtk_tree_view_set_search_column(GTK_TREE_VIEW(treeview), 1);
+    treeview = audgui_list_new (& callbacks, NULL, 0);
+    audgui_list_add_column (treeview, NULL, 0, G_TYPE_INT, 7);
+    audgui_list_add_column (treeview, NULL, 1, G_TYPE_STRING, -1);
 
     g_signal_connect (gtk_tree_view_get_selection ((GtkTreeView *) treeview),
      "changed", (GCallback) selection_changed, NULL);
@@ -352,7 +349,7 @@ static void create_window (void)
 
 void audgui_jump_to_track (void)
 {
-    gboolean create = (! jump_to_track_win);
+    bool_t create = (! jump_to_track_win);
     if (create)
         create_window ();
 
