@@ -1,20 +1,20 @@
-/*  Audacious - Cross-platform multimedia player
- *  Copyright (C) 2005-2011  Audacious development team.
+/*
+ * ui_playlist_manager.c
+ * Copyright 2006-2011 Giacomo Lozito and John Lindgren
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; under version 3 of the License.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * 1. Redistributions of source code must retain the above copyright notice,
+ *    this list of conditions, and the following disclaimer.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses>.
+ * 2. Redistributions in binary form must reproduce the above copyright notice,
+ *    this list of conditions, and the following disclaimer in the documentation
+ *    provided with the distribution.
  *
- *  The Audacious team does not consider modular code linking to
- *  Audacious or using our public API to be a derived work.
+ * This software is provided "as is" and without any warranty, express or
+ * implied. In no event shall the authors be liable for any damages arising from
+ * the use of this software.
  */
 
 #include <gtk/gtk.h>
@@ -26,6 +26,7 @@
 #include <libaudcore/hook.h>
 
 #include "config.h"
+#include "init.h"
 #include "libaudgui.h"
 #include "libaudgui-gtk.h"
 #include "list.h"
@@ -34,7 +35,7 @@ static GtkWidget * playman_win = NULL;
 
 static void save_position (GtkWidget * window)
 {
-    gint x, y, w, h;
+    int x, y, w, h;
     gtk_window_get_position ((GtkWindow *) window, & x, & y);
     gtk_window_get_size ((GtkWindow *) window, & w, & h);
 
@@ -44,7 +45,7 @@ static void save_position (GtkWidget * window)
     aud_set_int ("audgui", "playlist_manager_h", h);
 }
 
-static gboolean hide_cb (GtkWidget * window)
+static bool_t hide_cb (GtkWidget * window)
 {
     save_position (window);
     gtk_widget_hide (window);
@@ -73,12 +74,14 @@ static void save_config_cb (void * hook_data, void * user_data)
         save_position ((GtkWidget *) user_data);
 }
 
-static void get_value (void * user, gint row, gint column, GValue * value)
+static void get_value (void * user, int row, int column, GValue * value)
 {
     switch (column)
     {
-    case 0:
-        g_value_take_string (value, aud_playlist_get_title (row));
+    case 0:;
+        char * title = aud_playlist_get_title (row);
+        g_value_set_string (value, title);
+        str_unref (title);
         break;
     case 1:
         g_value_set_int (value, aud_playlist_entry_count (row));
@@ -86,22 +89,22 @@ static void get_value (void * user, gint row, gint column, GValue * value)
     }
 }
 
-static gboolean get_selected (void * user, gint row)
+static bool_t get_selected (void * user, int row)
 {
     return (row == aud_playlist_get_active ());
 }
 
-static void set_selected (void * user, gint row, gboolean selected)
+static void set_selected (void * user, int row, bool_t selected)
 {
     if (selected)
         aud_playlist_set_active (row);
 }
 
-static void select_all (void * user, gboolean selected)
+static void select_all (void * user, bool_t selected)
 {
 }
 
-static void activate_row (void * user, gint row)
+static void activate_row (void * user, int row)
 {
     aud_playlist_set_active (row);
 
@@ -109,7 +112,7 @@ static void activate_row (void * user, gint row)
         hide_cb (playman_win);
 }
 
-static void shift_rows (void * user, gint row, gint before)
+static void shift_rows (void * user, int row, int before)
 {
     if (before < row)
         aud_playlist_reorder (row, before, 1);
@@ -129,26 +132,26 @@ static const AudguiListCallbacks callbacks = {
  .get_data = NULL,
  .receive_data = NULL};
 
-static gboolean search_cb (GtkTreeModel * model, gint column, const gchar * key,
+static bool_t search_cb (GtkTreeModel * model, int column, const char * key,
  GtkTreeIter * iter, void * user)
 {
     GtkTreePath * path = gtk_tree_model_get_path (model, iter);
     g_return_val_if_fail (path, TRUE);
-    gint row = gtk_tree_path_get_indices (path)[0];
+    int row = gtk_tree_path_get_indices (path)[0];
     gtk_tree_path_free (path);
 
-    gchar * temp = aud_playlist_get_title (row);
+    char * temp = aud_playlist_get_title (row);
     g_return_val_if_fail (temp, TRUE);
-    gchar * title = g_utf8_strdown (temp, -1);
-    g_free (temp);
+    char * title = g_utf8_strdown (temp, -1);
+    str_unref (temp);
 
     temp = g_utf8_strdown (key, -1);
-    gchar * * keys = g_strsplit (temp, " ", 0);
+    char * * keys = g_strsplit (temp, " ", 0);
     g_free (temp);
 
-    gboolean match = FALSE;
+    bool_t match = FALSE;
 
-    for (gint i = 0; keys[i]; i ++)
+    for (int i = 0; keys[i]; i ++)
     {
         if (! keys[i][0])
             continue;
@@ -168,16 +171,16 @@ static gboolean search_cb (GtkTreeModel * model, gint column, const gchar * key,
     return ! match; /* TRUE == not matched, FALSE == matched */
 }
 
-static gboolean position_changed = FALSE;
-static gboolean playlist_activated = FALSE;
+static bool_t position_changed = FALSE;
+static bool_t playlist_activated = FALSE;
 
 static void update_hook (void * data, void * list)
 {
-    gint rows = aud_playlist_count ();
+    int rows = aud_playlist_count ();
 
     if (GPOINTER_TO_INT (data) == PLAYLIST_UPDATE_STRUCTURE)
     {
-        gint old_rows = audgui_list_row_count (list);
+        int old_rows = audgui_list_row_count (list);
 
         if (rows < old_rows)
             audgui_list_delete_rows (list, rows, old_rows - rows);
@@ -254,10 +257,10 @@ void audgui_playlist_manager (void)
     gtk_window_set_geometry_hints( GTK_WINDOW(playman_win) , GTK_WIDGET(playman_win) ,
                                    &playman_win_hints , GDK_HINT_MIN_SIZE );
 
-    gint x = aud_get_int ("audgui", "playlist_manager_x");
-    gint y = aud_get_int ("audgui", "playlist_manager_y");
-    gint w = aud_get_int ("audgui", "playlist_manager_w");
-    gint h = aud_get_int ("audgui", "playlist_manager_h");
+    int x = aud_get_int ("audgui", "playlist_manager_x");
+    int y = aud_get_int ("audgui", "playlist_manager_y");
+    int w = aud_get_int ("audgui", "playlist_manager_w");
+    int h = aud_get_int ("audgui", "playlist_manager_h");
 
     if (w && h)
     {
@@ -273,8 +276,8 @@ void audgui_playlist_manager (void)
     gtk_container_add( GTK_CONTAINER(playman_win) , playman_vbox );
 
     playman_pl_lv = audgui_list_new (& callbacks, NULL, aud_playlist_count ());
-    audgui_list_add_column (playman_pl_lv, _("Title"), 0, G_TYPE_STRING, TRUE);
-    audgui_list_add_column (playman_pl_lv, _("Entries"), 1, G_TYPE_INT, FALSE);
+    audgui_list_add_column (playman_pl_lv, _("Title"), 0, G_TYPE_STRING, -1);
+    audgui_list_add_column (playman_pl_lv, _("Entries"), 1, G_TYPE_INT, 7);
     audgui_list_set_highlight (playman_pl_lv, aud_playlist_get_playing ());
     gtk_tree_view_set_search_equal_func ((GtkTreeView *) playman_pl_lv,
      search_cb, NULL, NULL);
@@ -329,3 +332,17 @@ void audgui_playlist_manager (void)
 
     hook_associate ("config save", save_config_cb, playman_win);
 }
+
+void audgui_playlist_manager_cleanup (void)
+{
+    if (! playman_win)
+        return;
+
+    hook_dissociate ("playlist update", update_hook);
+    hook_dissociate ("playlist activate", activate_hook);
+    hook_dissociate ("playlist set playing", position_hook);
+    hook_dissociate ("config save", save_config_cb);
+
+    gtk_widget_destroy (playman_win);
+    playman_win = NULL;
+}
